@@ -83,6 +83,12 @@ STAGES = ['input', 'cram', 'gvcf', 'joint_calling', 'annotate', 'load_to_es']
     multiple=True,
     help='Don\'t process specified samples. Can be set multiple times.',
 )
+@click.option(
+    '--force-sample',
+    'force_sample',
+    multiple=True,
+    help='Force reprocessing these samples. Can be set multiple times.',
+)
 @click.option(
     '--output-version',
     'output_version',
@@ -153,6 +159,7 @@ def main(
     start_from_stage: str,
     end_with_stage: str,
     skip_samples: Collection[str],
+    force_sample: Collection[str],
     output_version: str,
     keep_scratch: bool,
     overwrite: bool,
@@ -239,6 +246,7 @@ def main(
         start_from_stage=start_from_stage,
         end_with_stage=end_with_stage,
         skip_samples=skip_samples,
+        force_sample=force_sample,
         use_gnarly=use_gnarly,
         use_as_vqsr=use_as_vqsr,
         hc_shards_num=hc_shards_num,
@@ -266,6 +274,7 @@ def _add_jobs(  # pylint: disable=too-many-statements
     start_from_stage: Optional[str],
     end_with_stage: Optional[str],
     skip_samples: Collection[str],
+    force_sample: Collection[str],
     use_gnarly: bool,
     use_as_vqsr: bool,
     hc_shards_num: int,
@@ -313,6 +322,13 @@ def _add_jobs(  # pylint: disable=too-many-statements
             analysis_type='gvcf',
             analysis_project=analysis_project,
         )
+        for sid, a in gvcf_analysis_per_sid.items():
+            a.output = a.output.replace('-test/', f'-{output_suffix}/')
+            a.output = a.output.replace('-main/', f'-{output_suffix}/')
+        for sid, a in cram_analysis_per_sid.items():
+            a.output = a.output.replace('-test/', f'-{output_suffix}/')
+            a.output = a.output.replace('-main/', f'-{output_suffix}/')
+
         seq_info_by_sid = SMDB.find_seq_info_by_sid(sample_ids)
 
         for s in samples:
@@ -332,9 +348,11 @@ def _add_jobs(  # pylint: disable=too-many-statements
                 check_existence=check_inputs_existence,
             )
             cram_job = None
-            if not found_cram_path:
+            if not found_cram_path or s['id'] in force_sample:
                 if skip_cram_stage:
                     continue
+                if s['id'] in force_sample:
+                    logger.info(f'Force rerunning sample {s["id"]}')
                 seq_info = seq_info_by_sid[s['id']]
                 logger.info(f'Checking sequence.meta in {seq_info}:')
                 alignment_input = sm_get_reads_data(
@@ -375,9 +393,11 @@ def _add_jobs(  # pylint: disable=too-many-statements
                 skip_stage=skip_gvcf_stage,
                 check_existence=check_inputs_existence,
             )
-            if not found_gvcf_path:
+            if not found_gvcf_path or s['id'] in force_sample:
                 if skip_gvcf_stage:
                     continue
+                if s['id'] in force_sample:
+                    logger.info(f'Force rerunning sample {s["id"]}')
                 if hc_intervals_j is None and hc_shards_num > 1:
                     hc_intervals_j = _add_split_intervals_job(
                         b=b,
@@ -479,7 +499,6 @@ def _add_jobs(  # pylint: disable=too-many-statements
         output_path=expected_vqsr_site_only_vcf_path,
         is_small_callset=is_small_callset,
         is_huge_callset=is_huge_callset,
-        n_samples=len(good_samples),
         depends_on=[jc_job],
         intervals=intervals_j.intervals,
         joint_calling_tmp_bucket=joint_calling_tmp_bucket,
@@ -502,14 +521,14 @@ def _add_jobs(  # pylint: disable=too-many-statements
     annotated_mt_path = f'{anno_tmp_bucket}/combined.mt'
     checkpoints_bucket = f'{anno_tmp_bucket}/checkpoints'
     if skip_anno_stage:
-        annotate_job = None
+        annotate_combined_j = None
     else:
         if utils.can_reuse(annotated_mt_path, overwrite):
-            annotate_job = b.new_job(f'Make MT and annotate [reuse]')
+            annotate_combined_j = b.new_job(f'Make MT and annotate [reuse]')
         else:
-            annotate_job = dataproc.hail_dataproc_job(
+            annotate_combined_j = dataproc.hail_dataproc_job(
                 b,
-                f'batch_seqr_loader/scripts/make_annotated_mt.py '
+                f'batch_seqr_loader/scripts/vcf_to_mt.py '
                 f'--vcf-path {expected_jc_vcf_path} '
                 f'--site-only-vqsr-vcf-path {expected_vqsr_site_only_vcf_path} '
                 f'--dest-mt-path {annotated_mt_path} '
@@ -521,7 +540,7 @@ def _add_jobs(  # pylint: disable=too-many-statements
                 max_age='16h',
                 packages=utils.DATAPROC_PACKAGES,
                 num_secondary_workers=utils.NUMBER_OF_DATAPROC_WORKERS,
-                job_name=f'Annotating',
+                job_name=f'Annotate joint-called callset',
                 vep='GRCh38',
                 depends_on=[vqsr_job] if vqsr_job else [],
             )
@@ -533,38 +552,41 @@ def _add_jobs(  # pylint: disable=too-many-statements
     for proj_name in output_projects:
         samples = samples_by_project.get(proj_name)
         proj_name = proj_name if output_suffix == 'main' else f'{proj_name}-test'
-        proj_tmp_bucket = f'{anno_tmp_bucket}/projects/{proj_name}'
+        proj_bucket = f'gs://cpg-{proj_name}-{output_suffix}'
+        proj_tmp_bucket = f'gs://cpg-{proj_name}-{output_suffix}-tmp'
+        project_mt_path = f'{proj_bucket}/mt/{proj_name}.mt'
         # Make a list of project samples to subset from the entire matrix table
         sample_ids = [s['id'] for s in samples]
-        subset_path = f'{proj_tmp_bucket}/samples.txt'
+        subset_path = f'{proj_tmp_bucket}/seqr-samples.txt'
         with hl.hadoop_open(subset_path, 'w') as f:
             f.write('\n'.join(sample_ids))
 
-        remap_path = f'{proj_tmp_bucket}/external_id_map.tsv'
-        with hl.hadoop_open(remap_path, 'w') as f:
-            f.write('\t'.join(['s', 'seqr_id']) + '\n')
-            for s in samples:
-                f.write('\t'.join([s['id'], s['external_id']]) + '\n')
+        annotate_project_j = dataproc.hail_dataproc_job(
+            b,
+            f'batch_seqr_loader/scripts/mt_to_projectmt.py '
+            f'--mt-path {annotated_mt_path} '
+            f'--out-mt-path {project_mt_path}'
+            f'--subset-tsv {subset_path}',
+            max_age='8h',
+            packages=utils.DATAPROC_PACKAGES,
+            num_secondary_workers=utils.NUMBER_OF_DATAPROC_WORKERS,
+            job_name=f'{proj_name}: annotate project',
+            depends_on=[annotate_combined_j],
+        )
 
         timestamp = time.strftime("%Y%m%d-%H%M%S")
         dataproc.hail_dataproc_job(
             b,
-            f'batch_seqr_loader/scripts/load_project_to_es.py '
-            f'--mt-path {annotated_mt_path} '
-            '--make-checkpoints '
-            + ('--overwrite ' if overwrite else '')
-            + f'--bucket {proj_tmp_bucket} '
+            f'batch_seqr_loader/scripts/projectmt_to_es.py '
+            f'--mt-path {project_mt_path} '
             f'--es-index {proj_name}-{output_version}-{timestamp} '
             f'--es-index-min-num-shards 1 '
-            f'--subset-tsv {subset_path} '
-            f'--remap-tsv {remap_path} '
-            f'--genome-version GRCh38 '
             f'{"--prod" if prod else ""}',
             max_age='16h',
             packages=utils.DATAPROC_PACKAGES,
             num_secondary_workers=10,
-            job_name=f'{proj_name}: annotate project and create ES index',
-            depends_on=[annotate_job],
+            job_name=f'{proj_name}: create ES index',
+            depends_on=[annotate_project_j],
             scopes=['cloud-platform'],
         )
     return b
@@ -725,7 +747,7 @@ def _make_realign_jobs(
         assert alignment_input.index_path
         assert not alignment_input.fqs1 and not alignment_input.fqs2
         j.storage(
-            '400G' if alignment_input.bam_or_cram_path.endswith('.cram') else '1000G'
+            '350G' if alignment_input.bam_or_cram_path.endswith('.cram') else '500G'
         )
 
         if alignment_input.bam_or_cram_path.startswith('gs://'):
@@ -1168,6 +1190,7 @@ def _make_joint_genotype_jobs(
         sample_names_to_add,
         sample_names_will_be_in_db,
         sample_names_already_added,
+        sample_names_to_remove,
         updating_existing_db,
         sample_map_bucket_path,
     ) = _samples_to_add_to_db(
@@ -1191,6 +1214,7 @@ def _make_joint_genotype_jobs(
                 genomicsdb_gcs_path=genomicsdb_path_per_interval[idx],
                 sample_names_to_add=sample_names_to_add,
                 sample_names_to_skip=sample_names_already_added,
+                sample_names_to_remove=sample_names_to_remove,
                 sample_names_will_be_in_db=sample_names_will_be_in_db,
                 updating_existing_db=updating_existing_db,
                 sample_map_bucket_path=sample_map_bucket_path,
@@ -1264,7 +1288,6 @@ def _make_vqsr_jobs(
     gathered_vcf_path: str,
     is_small_callset: bool,
     is_huge_callset: bool,
-    n_samples: int,
     output_path: str,
     depends_on,
     intervals,
@@ -1280,7 +1303,6 @@ def _make_vqsr_jobs(
             input_vcf_gathered=gathered_vcf_path,
             is_small_callset=is_small_callset,
             is_huge_callset=is_huge_callset,
-            n_samples=n_samples,
             work_bucket=tmp_vqsr_bucket,
             web_bucket=tmp_vqsr_bucket,
             depends_on=depends_on,
@@ -1444,7 +1466,7 @@ def _samples_to_add_to_db(
     samples,
     tmp_bucket: str,
     gvcf_by_sid: Dict[str, str],
-) -> Tuple[Set[str], Set[str], Set[str], bool, str]:
+) -> Tuple[Set[str], Set[str], Set[str], Set[str], bool, str]:
     if utils.file_exists(join(genomicsdb_gcs_path, 'callset.json')):
         # Checking if samples exists in the DB already
         genomicsdb_metadata = join(local_tmp_dir, f'callset-{interval_idx}.json')
@@ -1503,6 +1525,7 @@ def _samples_to_add_to_db(
         sample_names_already_added = set()
         sample_names_to_add = {s['id'] for s in samples}
         sample_names_will_be_in_db = sample_names_to_add
+        sample_names_to_remove = set()
         updating_existing_db = False
         logger.info(
             f'GenomicDB {genomicsdb_gcs_path} doesn\'t exist, so creating a new one '
@@ -1520,6 +1543,7 @@ def _samples_to_add_to_db(
         sample_names_to_add,
         sample_names_will_be_in_db,
         sample_names_already_added,
+        sample_names_to_remove,
         updating_existing_db,
         sample_map_bucket_path,
     )
@@ -1530,6 +1554,7 @@ def _add_import_gvcfs_job(
     genomicsdb_gcs_path: str,
     sample_names_to_add: Set[str],
     sample_names_to_skip: Set[str],
+    sample_names_to_remove: Set[str],
     sample_names_will_be_in_db: Set[str],
     updating_existing_db: bool,
     sample_map_bucket_path: str,
@@ -1542,14 +1567,30 @@ def _add_import_gvcfs_job(
     Add GVCFs to a genomics database (or create a new instance if it doesn't exist)
     Returns a Job, or None if no new samples to add
     """
+    rm_cmd = ''
+    msg = ''
+
     if updating_existing_db:
         # Update existing DB
         genomicsdb_param = f'--genomicsdb-update-workspace-path {genomicsdb_gcs_path}'
         job_name = 'Adding to GenomicsDB'
+        msg = f'Adding {len(sample_names_to_add)} samples: {", ".join(sample_names_to_add)}'
+        if sample_names_to_skip:
+            msg += (
+                f'Skipping adding {len(sample_names_to_skip)} samples that are already '
+                f'in the DB: {", ".join(sample_names_to_skip)}"'
+            )
     else:
         # Initiate new DB
         genomicsdb_param = f'--genomicsdb-workspace-path {genomicsdb_gcs_path}'
         job_name = 'Creating GenomicsDB'
+        if sample_names_to_remove:
+            # Need to remove the existing database
+            rm_cmd = f'gsutil -q rm -rf {genomicsdb_gcs_path}'
+        msg = (
+            f'Creating a new DB with {len(sample_names_to_add)} samples: '
+            f'{", ".join(sample_names_to_add)}'
+        )
 
     sample_map = b.read_input(sample_map_bucket_path)
 
@@ -1590,9 +1631,9 @@ def _add_import_gvcfs_job(
     export GOOGLE_APPLICATION_CREDENTIALS=/gsa-key/key.json
     gcloud -q auth activate-service-account --key-file=$GOOGLE_APPLICATION_CREDENTIALS
 
-    echo "Adding {len(sample_names_to_add)} samples: {', '.join(sample_names_to_add)}"
-    {f'echo "Skipping adding {len(sample_names_to_skip)} samples that are already in the DB: '
-     f'{", ".join(sample_names_to_skip)}"' if sample_names_to_skip else ''}
+    echo "{msg}"
+    
+    {rm_cmd}
 
     gatk --java-options -Xms{java_mem}g \\
       GenomicsDBImport \\
